/*
 * main module for VGA demo.
 *
 * (C) Arlet Ottens
 */
module main( Clk, Reset, R, G, B, VSync, HSync, SW0, SW1, SW2 );

input Clk;
input Reset;

// VGA
output R;
output G;
output B;
output VSync;
output HSync;

// Switches
input SW0;
input SW1;
input SW2;

// definitions
reg R;
reg G;
reg B;
reg VSync;
reg HSync;

wire HSync0;
wire VSync0;				 
wire [9:0] Col;
wire [8:0] Row;
wire Col0;
wire Row0;
wire screen_active;
wire PixelClock;

vga vga( .Reset(Reset), 
	 .HSync(HSync0), 
	 .VSync(VSync0), 
	 .Clk50(Clk), 
	 .PClk(PixelClock), 
	 .Col(Col), 
	 .Row(Row), 
	 .Col0(Col0), 
	 .Row0(Row0), 
	 .Active(screen_active) );

reg [17:0] X0, Y0, X1, Y1, XI;
reg [17:0] RX0, RY0, RX1, RY1;				    
reg [17:0] GX0, GY0, GX1, GY1;				    
reg signed [17:0] DY, DX;
reg signed [17:0] RDY, RDX;
reg signed [17:0] GDY, GDX;
reg do_DY;

/*
 * Rotation per line/frame depending on SW2
 */

wire Rot = SW2 ? Row0 : Col0;
				
/*
 * maintain transformation parameters
 */
always @(posedge Clk) begin
    if( Reset ) begin
	DY <= 2000;
	DX <= 2000;
	do_DY <= 0;
    end else if( Rot ) begin
	DY <= DY - (DX >>> 8);
	do_DY <= 1;
    end else if( do_DY ) begin
	DX <= DX + (DY >>> 8);
	do_DY <= 0;
    end

    if( Reset ) begin
	RDY <= 2050;
	RDX <= 1950;
    end else if( Rot ) begin
	RDY <= RDY - (RDX >>> 8);
    end else if( do_DY ) begin
	RDX <= RDX + (RDY >>> 8);
    end

    if( Reset ) begin
	GDY <= 2000;
	GDX <= 2000;
    end else if( Rot ) begin
	GDY <= GDY - (GDX >>> 8);
    end else if( do_DY ) begin
	GDX <= GDX + (GDY >>> 8);
    end
end

/*
 * calculate per-pixel mapping (based on rotozoom). Multipliers can be 
 * avoided by more clever code, but the Spartan-3 has plenty for this 
 * purpose.
 */
always @(posedge Clk) begin
    if( PixelClock ) begin
	if( Row0 ) begin
	    X0 <= -DX * 18'd319;
	    Y0 <=  DY * 18'd241;
	    X1 <= -DX * 18'd319;
	    Y1 <=  DY * 18'd241;
	end else if( Col0 ) begin
	    X0 <= X1 + DY;
	    Y0 <= Y1 - DX;
	    X1 <= X1 + DY;
	    Y1 <= Y1 - DX;
	end else begin
	    X0 <= X0 + DX;
	    Y0 <= Y0 + DY;
	end

	if( Row0 ) begin
	    RX0 <= -RDX * 18'd319;
	    RY0 <=  RDY * 18'd231;
	    RX1 <= -RDX * 18'd319;
	    RY1 <=  RDY * 18'd237;
	end else if( Col0 ) begin
	    RX0 <= RX1 + RDY;
	    RY0 <= RY1 - RDX;
	    RX1 <= RX1 + RDY;
	    RY1 <= RY1 - RDX;
	end else begin
	    RX0 <= RX0 + RDX;
	    RY0 <= RY0 + RDY;
	end

	if( Row0 ) begin
	    GX0 <= -GDX * 18'd300;
	    GY0 <=  GDY * 18'd221;
	    GX1 <= -GDX * 18'd300;
	    GY1 <=  GDY * 18'd221;
	end else if( Col0 ) begin
	    GX0 <= GX1 + GDY;
	    GY0 <= GY1 - GDX;
	    GX1 <= GX1 + GDY;
	    GY1 <= GY1 - GDX;
	end else begin
	    GX0 <= GX0 + GDX;
	    GY0 <= GY0 + GDY;
	end
    end
end

/*
 * select bits to XOR based on SW0 
 */
wire [17:0] SRX1 = SW0 ? RX0 : RX0 << 1;
wire [17:0] SGX1 = SW0 ? GX0 : GX0 << 1;
wire [17:0]  SX1 = SW0 ?  X0 :  X0 << 1;

wire [17:0] SRY1 = SW0 ? RY0 : RY0 << 1;
wire [17:0] SGY1 = SW0 ? GY0 : GY0 << 1;
wire [17:0]  SY1 = SW0 ?  Y0 :  Y0 << 1;

/*
 * select squares or lines based on SW1
 */
wire RB = SW1 ? (SRX1[17] ^ SRY1[17]) : SRX1[17];
wire GB = SW1 ? (SGX1[17] ^ SGY1[17]) : SGX1[17];
wire BB = SW1 ? ( SX1[17] ^  SY1[17])  : SX1[17];

/*
 * output HSync/VSync and RGB data
 */
always @(posedge Clk) begin
    HSync <= HSync0;
    VSync <= VSync0;
    if( screen_active ) begin
	R <= ~BB & (RB | (GB & ~RB));
	G <= GB & ~BB & ~RB;
	B <= BB;
    end else begin 
	B <= 0;
	G <= 0;
	R <= 0;
    end
end

endmodule
